window.onload = function () {
    document.querySelector("#connect-dev").onclick = connectDevice;
}

const radarServiceUuid = "96d812ad-3dbc-4bd5-9fed-e105f7c28d77";
const thresholdCharacteristicsUuid = "58044a1e-23cc-4206-ae76-94ecb0c7e44c";
const holdtimeCharacteristicsUuid = "6c3415df-4817-44af-9438-3a1468c270aa";

function AutomaticCooler(device, server, thresholdCharacteristics, holdtimeCharacteristics) {
	this.deviceNumber = AutomaticCooler.nextDeviceNumber++;

    this.server = server;
    this.thresholdCharacteristics = thresholdCharacteristics;
    this.holdtimeCharacteristics = holdtimeCharacteristics;

    this.deviceElement = document.querySelector("#device-template").cloneNode(true);
	this.deviceElement.classList.remove("d-none");

    this.title = this.deviceElement.querySelector(".card-title");
    this.thresholdInput = this.deviceElement.querySelector(".threshold-rng");
    this.holdtimeInput = this.deviceElement.querySelector(".holdtime-rng");
    this.thresholdIndicator = this.deviceElement.querySelector(".threshold-indicator");
    this.holdtimeIndicator = this.deviceElement.querySelector(".holdtime-indicator");
	this.disconnectButton = this.deviceElement.querySelector(".disconnect-btn");
	this.saveButton = this.deviceElement.querySelector(".save-btn");
	this.saveSpinner = this.deviceElement.querySelector(".save-spinner");

	this.saveButton.onclick = this.saveValues.bind(this);
	this.disconnectButton.onclick = this.disconnectDevice.bind(this);
	this.thresholdInput.oninput = this.updateIndicators.bind(this);
	this.thresholdInput.onchange = this.updateIndicators.bind(this);
	this.holdtimeInput.oninput = this.updateIndicators.bind(this);
	this.holdtimeInput.onchange = this.updateIndicators.bind(this);

	this.title.textContent = "Automatic Cooler #" + this.deviceNumber;
}

AutomaticCooler.prototype.loadValues = async function () {
    var currentThreshold = await this.thresholdCharacteristics.readValue();
    var currentHoldTime = await this.holdtimeCharacteristics.readValue();

    var currentThresholdVal = currentThreshold.getUint8(0) | (currentThreshold.getUint8(1) << 8);
    var currentHoldtimeVal = currentHoldTime.getUint8(0) | (currentHoldTime.getUint8(1) << 8);

    this.thresholdInput.value = currentThresholdVal;
    this.holdtimeInput.value = currentHoldtimeVal;

	this.updateIndicators();
}

AutomaticCooler.prototype.updateIndicators = function () {
	this.thresholdIndicator.textContent = this.thresholdInput.value;
	this.holdtimeIndicator.textContent = this.holdtimeInput.value;
}

AutomaticCooler.prototype.saveValues = async function () {
	this.saveSpinner.classList.remove("d-none");

	var threshold = Uint8Array.of(this.thresholdInput.value & 0xFF, (this.thresholdInput.value & 0xFF00) >> 8);
	await this.thresholdCharacteristics.writeValueWithResponse(threshold);

	var holdtime = Uint8Array.of(this.holdtimeInput.value & 0xFF, (this.holdtimeInput.value & 0xFF00) >> 8);
	await this.holdtimeCharacteristics.writeValueWithResponse(holdtime);

	this.saveSpinner.classList.add("d-none");
}

AutomaticCooler.prototype.disconnectDevice = async function () {
	this.server.disconnect();
	this.deviceElement.parentElement.removeChild(this.deviceElement);
}

AutomaticCooler.nextDeviceNumber = 1; 

var devices = [];

async function connectDevice() {
    var device = await navigator.bluetooth.requestDevice({
        filters: [
            { name: "Automatic Cooler" },
        ],
        optionalServices: [radarServiceUuid]
    });
    var server = await device.gatt.connect();
    var radarService = await server.getPrimaryService(radarServiceUuid);
    var thresholdCharacteristics = await radarService.getCharacteristic(thresholdCharacteristicsUuid);
    var holdtimeCharacteristics = await radarService.getCharacteristic(holdtimeCharacteristicsUuid);

    var dev = new AutomaticCooler(device, server, thresholdCharacteristics, holdtimeCharacteristics);
    await dev.loadValues();
    document.querySelector("#devices").appendChild(dev.deviceElement);
    devices.push(dev);
}